@@ -47,10 +47,101 @@ def get_github_credentials():
         return token
 
 
-class GithubHandler(urllib2.BaseHandler):
+def isPrivate(urlPath, repos):
+    """Check if a URL is a private URL
 
-    """This handler creates a post request with login and token, see
-    http://github.com/blog/170-token-authentication for details"""
+    >>> isPrivate('/downloads/me/', None)
+    True
+    >>> isPrivate('/downloads/me/', [])
+    False
+    >>> isPrivate('/downloads/me/', ['me'])
+    True
+    >>> isPrivate('/repos/me/', ['me'])
+    True
+    >>> isPrivate('/downloads/me/', ['unknown'])
+    False
+    >>> isPrivate('/somewhere/me/', ['me'])
+    False
+    """
+    # If provided check whitelist of Github repos in the format
+    # "<userororg>/<repo>", for either API v3 or static downloads
+    if repos is None:
+        # for backward compatibility all repos are private if there is no
+        # "github-repos" entry in the buildout section.
+        return True
+    # now check the whitelist
+    for repo in repos:
+        if not repo:
+            # ignore empty entries
+            continue
+        api_repo = "/repos/%s/" % (repo,)
+        dl_repo = "/downloads/%s/" % (repo,)
+        if api_repo in urlPath or dl_repo in urlPath:
+            return True
+    return False
+
+
+class GithubHandler(urllib2.BaseHandler):
+    """This handler creates a post request with login and token
+
+    see http://github.com/blog/170-token-authentication for details
+
+    With a none github url the resulting request is unchanged::
+
+    >>> req = urllib2.Request('http://example.com/downloads/me/')
+    >>> handler = GithubHandler('--mytoken--')
+    >>> res = handler.https_request(req)
+    >>> res.get_full_url()
+    'http://example.com/downloads/me/'
+    >>> res is req
+    True
+
+    With a github url we get the access token::
+
+    >>> req = urllib2.Request('https://github.com/downloads/me/')
+    >>> res = handler.https_request(req)
+    >>> res.get_full_url()
+    'https://github.com/downloads/me/?access_token=--mytoken--'
+
+    If we provide an empty whitelist we get all github requests without a
+    token::
+
+    >>> handler = GithubHandler('--mytoken--', [])
+    >>> req = urllib2.Request('https://github.com/downloads/me/')
+    >>> res = handler.https_request(req)
+    >>> res.get_full_url()
+    'https://github.com/downloads/me/'
+
+    If the repository is in the whitelist is receives a token::
+
+    >>> handler = GithubHandler('--mytoken--', ['me'])
+    >>> req = urllib2.Request('https://github.com/downloads/me/')
+    >>> res = handler.https_request(req)
+    >>> res.get_full_url()
+    'https://github.com/downloads/me/?access_token=--mytoken--'
+
+    >>> req = urllib2.Request('https://github.com/downloads/me/?a=1&b=2')
+    >>> res = handler.https_request(req)
+    >>> res.get_full_url()
+    'https://github.com/downloads/me/?a=1&b=2&access_token=--mytoken--'
+
+    If no timeout is set in the original request, the timeout is set to 60::
+
+    >>> hasattr(req, 'timeout')
+    False
+    >>> res.timeout
+    60
+
+
+    The timeout from the original request is preseverd in the result::
+
+    >>> req = urllib2.Request('https://github.com/downloads/me/?a=1&b=2')
+    >>> req.timeout = 42
+    >>> res = handler.https_request(req)
+    >>> res.timeout
+    42
+
+    """
 
     def __init__(self, token, repos=None):
         self._token = token
@@ -59,33 +150,15 @@ class GithubHandler(urllib2.BaseHandler):
     def https_request(self, req):
         if req.get_method() == 'GET' and req.get_host().endswith('github.com'):
             url = req.get_full_url()
-            private = False
-
-            # If provided check whitelist of Github repos in the format
-            # "<userororg>/<repo>", for either API v3 or static downloads
-            if self._repos:
-                for repo in self._repos:
-                    api_repo = "/repos/%s/" % (repo,)
-                    dl_repo = "/downloads/%s/" % (repo,)
-                    if api_repo in url or dl_repo in url:
-                        private = True
-                        break
-            else:
-                private = True
-
-            if private:
-                log.debug("Found private github url %r", (url,))
-                data = urllib.urlencode(dict(access_token=self._token))
-                timeout = getattr(req, 'timeout', 60)
-                if hasattr(req, 'timeout'):
-                    timeout = req.timeout
-
-                # The GitHub v3 API requires a new URL.
-                scheme, netloc, path, params, query, fragment = \
+            scheme, netloc, path, params, query, fragment = \
                                         urlparse.urlparse(url)
-                query = '&'.join((query, data))
+            if isPrivate(path, self._repos):
+                log.debug("Found private github url %r", (url,))
+                token = urllib.urlencode(dict(access_token=self._token))
+                query = '&'.join([p for p in (query, token) if p])
                 new_url = urlparse.urlunparse((scheme, netloc, path, params,
                                                query, fragment))
+                timeout = getattr(req, 'timeout', 60)
                 req = urllib2.Request(new_url)
                 req.timeout = timeout
             else:
@@ -192,7 +265,7 @@ def install(buildout=None, pwd_path=None):
         if creds:
             new_handlers.append(auth_handler)
         if creds or github_creds:
-            download.url_opener = URLOpener(creds, github_creds)
+            download.url_opener = URLOpener(creds, github_creds, github_repos)
         if new_handlers:
             if urllib2._opener is not None:
                 handlers = urllib2._opener.handlers[:]
@@ -208,9 +281,10 @@ def install(buildout=None, pwd_path=None):
 
 class URLOpener(download.URLOpener):
 
-    def __init__(self, creds, github_creds):
+    def __init__(self, creds, github_creds, github_repos):
         self.creds = {}
         self.github_creds = github_creds
+        self.github_repos = github_repos
         for realm, uris, user, password in creds:
             parts = urlparse.urlparse(uris)
             self.creds[(parts[1], realm)] = (user, password)
@@ -220,7 +294,10 @@ class URLOpener(download.URLOpener):
         if self.github_creds and not data:
             scheme, netloc, path, params, query, fragment = urlparse.urlparse(
                 url)
-            if scheme == 'https' and netloc.endswith('github.com'):
+            if (scheme == 'https'
+                and netloc.endswith('github.com')
+                and isPrivate(url, self.github_repos)
+               ):
                 log.debug("Appending github credentials to url %r", url)
                 token = self.github_creds
                 cred = urllib.urlencode(dict(access_token=token))
